@@ -28,6 +28,7 @@ GraphicsContext currentcontext;
 
 #endif
 
+void (*__svgalib_nonaccel_fillbox)(int, int, int, int, int);
 static int screenoffset = 0;	/* Used by copy(box)toscreen. */
 
 
@@ -35,143 +36,143 @@ static int screenoffset = 0;	/* Used by copy(box)toscreen. */
 
 static framebufferfunctions ff8 =
 {
-    driver8_setpixel,
-    driver8_getpixel,
-    driver8_hline,
-    driver8_fillbox,
-    driver8_putbox,
-    driver8_getbox,
-    driver8_putboxmask,
-    driver8_putboxpart,
-    driver8_getboxpart,
-    driver8_copybox
+    __svgalib_driver8_setpixel,
+    __svgalib_driver8_getpixel,
+    __svgalib_driver8_hline,
+    __svgalib_driver8_fillbox,
+    __svgalib_driver8_putbox,
+    __svgalib_driver8_getbox,
+    __svgalib_driver8_putboxmask,
+    __svgalib_driver8_putboxpart,
+    __svgalib_driver8_getboxpart,
+    __svgalib_driver8_copybox
 };
 
 static framebufferfunctions ff16 =
 {
-    driver16_setpixel,
-    driver16_getpixel,
-    driver16_hline,
-    driver16_fillbox,
-    driver16_putbox,
-    driver16_getbox,
-    driver16_putboxmask,
-    driver16_putboxpart,
-    driver16_getboxpart,
-    driver16_copybox
+    __svgalib_driver16_setpixel,
+    __svgalib_driver16_getpixel,
+    __svgalib_driver16_hline,
+    __svgalib_driver16_fillbox,
+    __svgalib_driver16_putbox,
+    __svgalib_driver16_getbox,
+    __svgalib_driver16_putboxmask,
+    __svgalib_driver16_putboxpart,
+    __svgalib_driver16_getboxpart,
+    __svgalib_driver16_copybox
 };
 
 static framebufferfunctions ff24 =
 {
-    driver24_setpixel,
-    driver24_getpixel,
-    driver24_hline,
-    driver24_fillbox,
-    driver24_putbox,
-    driver24_getbox,
-    driver24_putboxmask,
-    driver24_putboxpart,
-    driver24_getboxpart,
-    driver24_copybox
+    __svgalib_driver24_setpixel,
+    __svgalib_driver24_getpixel,
+    __svgalib_driver24_hline,
+    __svgalib_driver24_fillbox,
+    __svgalib_driver24_putbox,
+    __svgalib_driver24_getbox,
+    __svgalib_driver24_putboxmask,
+    __svgalib_driver24_putboxpart,
+    __svgalib_driver24_getboxpart,
+    __svgalib_driver24_copybox
 };
 
 static framebufferfunctions ff32 =
 {
-    driver32_setpixel,
-    driver32_getpixel,
-    driver32_hline,
-    driver32_fillbox,
-    driver32_putbox,
-    driver32_getbox,
-    driver32_putboxmask,
-    driver32_putboxpart,
-    driver32_getboxpart,
-    driver32_copybox
+    __svgalib_driver32_setpixel,
+    __svgalib_driver32_getpixel,
+    __svgalib_driver32_hline,
+    __svgalib_driver32_fillbox,
+    __svgalib_driver32_putbox,
+    __svgalib_driver32_getbox,
+    __svgalib_driver32_putboxmask,
+    __svgalib_driver32_putboxpart,
+    __svgalib_driver32_getboxpart,
+    __svgalib_driver32_copybox
 };
 
 static framebufferfunctions ff8paged =
 {
-    driver8p_setpixel,
-    driver8p_getpixel,
-    driver8p_hline,
-    driver8p_fillbox,
-    driver8p_putbox,
-    driver8p_getbox,
-    driver8p_putboxmask,
-    driver8p_putboxpart,
-    driver8p_getboxpart,
-    driver8p_copybox
+    __svgalib_driver8p_setpixel,
+    __svgalib_driver8p_getpixel,
+    __svgalib_driver8p_hline,
+    __svgalib_driver8p_fillbox,
+    __svgalib_driver8p_putbox,
+    __svgalib_driver8p_getbox,
+    __svgalib_driver8p_putboxmask,
+    __svgalib_driver8p_putboxpart,
+    __svgalib_driver8p_getboxpart,
+    __svgalib_driver8p_copybox
 };
 
 static framebufferfunctions ff16paged =
 {
-    driver16p_setpixel,
-    driver16p_getpixel,
-    driver16p_hline,
-    driver16p_fillbox,
-    driver16p_putbox,
-    driver16p_getbox,
-    driver16p_putboxmask,
-    driver16p_putboxpart,
-    driver16p_getboxpart,
-    driver16p_copybox
+    __svgalib_driver16p_setpixel,
+    __svgalib_driver16p_getpixel,
+    __svgalib_driver16p_hline,
+    __svgalib_driver16p_fillbox,
+    __svgalib_driver16p_putbox,
+    __svgalib_driver16p_getbox,
+    __svgalib_driver16p_putboxmask,
+    __svgalib_driver16p_putboxpart,
+    __svgalib_driver16p_getboxpart,
+    __svgalib_driver16p_copybox
 };
 
 static framebufferfunctions ff24paged =
 {
-    driver24p_setpixel,
-    driver24p_getpixel,
-    driver24p_hline,
-    driver24p_fillbox,
-    driver24p_putbox,
-    driver24p_getbox,
-    driver24p_putboxmask,
-    driver24p_putboxpart,
-    driver24p_getboxpart,
-    driver24p_copybox
+    __svgalib_driver24p_setpixel,
+    __svgalib_driver24p_getpixel,
+    __svgalib_driver24p_hline,
+    __svgalib_driver24p_fillbox,
+    __svgalib_driver24p_putbox,
+    __svgalib_driver24p_getbox,
+    __svgalib_driver24p_putboxmask,
+    __svgalib_driver24p_putboxpart,
+    __svgalib_driver24p_getboxpart,
+    __svgalib_driver24p_copybox
 };
 
 static framebufferfunctions ff32paged =
 {
-    driver32p_setpixel,
-    driver32p_getpixel,
-    driver32p_hline,
-    driver32p_fillbox,
-    driver32p_putbox,
-    driver32p_getbox,
-    driver32p_putboxmask,
-    driver32p_putboxpart,
-    driver32p_getboxpart,
-    driver32p_copybox
+    __svgalib_driver32p_setpixel,
+    __svgalib_driver32p_getpixel,
+    __svgalib_driver32p_hline,
+    __svgalib_driver32p_fillbox,
+    __svgalib_driver32p_putbox,
+    __svgalib_driver32p_getbox,
+    __svgalib_driver32p_putboxmask,
+    __svgalib_driver32p_putboxpart,
+    __svgalib_driver32p_getboxpart,
+    __svgalib_driver32p_copybox
 };
 
 static framebufferfunctions ffplanar256 =
 {
-    (void *) driverplanar256_nothing,
-    (void *) driverplanar256_nothing,
-    (void *) driverplanar256_nothing,
-    (void *) driverplanar256_nothing,
-    driverplanar256_putbox,
-    (void *) driverplanar256_nothing,
-    (void *) driverplanar256_nothing,
-    (void *) driverplanar256_nothing,
-    (void *) driverplanar256_nothing,
-    (void *) driverplanar256_nothing,
+    (void *) __svgalib_driverplanar256_nothing,
+    (void *) __svgalib_driverplanar256_nothing,
+    (void *) __svgalib_driverplanar256_nothing,
+    (void *) __svgalib_driverplanar256_nothing,
+    __svgalib_driverplanar256_putbox,
+    (void *) __svgalib_driverplanar256_nothing,
+    (void *) __svgalib_driverplanar256_nothing,
+    (void *) __svgalib_driverplanar256_nothing,
+    (void *) __svgalib_driverplanar256_nothing,
+    (void *) __svgalib_driverplanar256_nothing,
 };
 
 #if 0				/* Not yet used */
 static framebufferfunctions ffplanar16 =
 {
-    (void *) driverplanar16_nothing,
-    (void *) driverplanar16_nothing,
-    (void *) driverplanar16_nothing,
-    (void *) driverplanar16_nothing,
-    (void *) driverplanar16_nothing,
-    (void *) driverplanar16_nothing,
-    (void *) driverplanar16_nothing,
-    (void *) driverplanar16_nothing,
-    (void *) driverplanar16_nothing,
-    (void *) driverplanar16_nothing,
+    (void *) __svgalib_driverplanar16_nothing,
+    (void *) __svgalib_driverplanar16_nothing,
+    (void *) __svgalib_driverplanar16_nothing,
+    (void *) __svgalib_driverplanar16_nothing,
+    (void *) __svgalib_driverplanar16_nothing,
+    (void *) __svgalib_driverplanar16_nothing,
+    (void *) __svgalib_driverplanar16_nothing,
+    (void *) __svgalib_driverplanar16_nothing,
+    (void *) __svgalib_driverplanar16_nothing,
+    (void *) __svgalib_driverplanar16_nothing,
 };
 #endif
 
@@ -297,12 +298,13 @@ int gl_setcontextvga(int m)
     }
     if (vga_getcurrentmode() == m) {
 	accelfuncs = vga_ext_set(VGA_EXT_AVAILABLE, VGA_AVAIL_ACCEL);
+	__svgalib_nonaccel_fillbox = __currentcontext.ff.driver_fillbox_func;
 	if (accelfuncs & ACCELFLAG_FILLBOX)
 	    __currentcontext.ff.driver_fillbox_func =
-		driver8a_fillbox;
+		__svgalib_driver8a_fillbox;
 	if (accelfuncs & ACCELFLAG_SCREENCOPY)
 	    __currentcontext.ff.driver_copybox_func =
-		driver8a_copybox;
+		__svgalib_driver8a_copybox;
     }
 #ifdef DLL_CONTEXT_SHADOW
     currentcontext = __currentcontext;
@@ -805,14 +807,14 @@ void gl_copyscreen(GraphicsContext * gc)
 	while (soffset / 4 < size) {
 	    int schunk, dchunk;
 	    int count;
-	    schunk = driver_setread(&__currentcontext, soffset, &svp);
-	    dchunk = driver_setwrite(gc, doffset, &dvp);
+	    schunk = __svgalib_driver_setread(&__currentcontext, soffset, &svp);
+	    dchunk = __svgalib_driver_setwrite(gc, doffset, &dvp);
 	    if (dchunk == 1) {
 		/* One byte left in segment. */
 		int pix;
 		pix = *(unsigned *) svp;	/* 32-bit pixel */
 		*(unsigned char *) dvp = pix;
-		dchunk = driver_setwrite(gc, doffset + 1, &dvp);
+		dchunk = __svgalib_driver_setwrite(gc, doffset + 1, &dvp);
 		*(unsigned short *) dvp = pix >> 8;
 		count = 1;	/* 1 pixel handled. */
 	    } else if (dchunk == 2) {
@@ -820,13 +822,13 @@ void gl_copyscreen(GraphicsContext * gc)
 		int pix;
 		pix = *(unsigned *) svp;	/* 32-bit pixel */
 		*(unsigned short *) dvp = pix;
-		dchunk = driver_setwrite(gc, doffset + 2, &dvp);
+		dchunk = __svgalib_driver_setwrite(gc, doffset + 2, &dvp);
 		*(unsigned char *) dvp = pix >> 16;
 		count = 1;	/* 1 pixel handled. */
 	    } else {
 		count = min(min(schunk / 4, dchunk / 3),
 			    size - (soffset / 4));
-		memcpy4to3(dvp, svp, count);
+		__svgalib_memcpy4to3(dvp, svp, count);
 	    }
 	    soffset += count * 4;
 	    doffset += count * 3;
@@ -841,10 +843,10 @@ void gl_copyscreen(GraphicsContext * gc)
 	while (soffset < size) {
 	    int schunk, dchunk;
 	    int count;
-	    schunk = driver_setread(&__currentcontext, soffset, &svp);
-	    dchunk = driver_setwrite(gc, doffset, &dvp);
+	    schunk = __svgalib_driver_setread(&__currentcontext, soffset, &svp);
+	    dchunk = __svgalib_driver_setwrite(gc, doffset, &dvp);
 	    count = min(min(schunk, dchunk), (size - soffset));
-	    memcpy32shift8(dvp, svp, count / 4);
+	    __svgalib_memcpy32shift8(dvp, svp, count / 4);
 	    soffset += count;
 	    doffset += count;
 	}
@@ -855,8 +857,8 @@ void gl_copyscreen(GraphicsContext * gc)
 	while (soffset < size) {
 	    int schunk, dchunk;
 	    int count;
-	    schunk = driver_setread(&__currentcontext, soffset, &svp);
-	    dchunk = driver_setwrite(gc, doffset, &dvp);
+	    schunk = __svgalib_driver_setread(&__currentcontext, soffset, &svp);
+	    dchunk = __svgalib_driver_setwrite(gc, doffset, &dvp);
 	    count = min(min(schunk, dchunk), (size - soffset));
 	    __memcpy(dvp, svp, count);
 	    soffset += count;
@@ -920,12 +922,12 @@ void gl_copyboxtocontext(int x1, int y1, int w, int h, GraphicsContext * gc,
 	    /* For paged modes or clipping, use another buffer. */
 	    void *buf2;
 	    buf2 = alloca(w * h * 3);
-	    memcpy4to3(buf2, buf, w * h);
+	    __svgalib_memcpy4to3(buf2, buf, w * h);
 	    gl_putbox(x2, y2 + screenoffset / BYTEWIDTH, w, h,
 		      buf2);
 	} else
 	    /* No clipping, linear. */
-	    driver24_putbox32(x2, y2, w, h, buf, w);
+	    __svgalib_driver24_putbox32(x2, y2, w, h, buf, w);
     } else			/* Contexts assumed to have same pixel size. */
 	gl_putbox(x2, y2 + screenoffset / BYTEWIDTH, w, h, buf);
 
@@ -971,12 +973,12 @@ void gl_copyboxfromcontext(GraphicsContext * gc, int x1, int y1, int w, int h,
 	    /* For paged modes or clipping, use another buffer. */
 	    void *buf2;
 	    buf2 = alloca(w * h * 3);
-	    memcpy4to3(buf2, buf, w * h);
+	    __svgalib_memcpy4to3(buf2, buf, w * h);
 	    gl_putbox(x2, y2 + screenoffset / BYTEWIDTH, w, h,
 		      buf2);
 	} else
 	    /* No clipping, linear. */
-	    driver24_putbox32(x2, y2, w, h, buf, w);
+	    __svgalib_driver24_putbox32(x2, y2, w, h, buf, w);
     } else			/* Contexts assumed to have same pixel size. */
 	gl_putbox(x2, y2 + screenoffset / BYTEWIDTH, w, h, buf);
 }
