@@ -8,6 +8,8 @@
  * Jan 1995:
  * Initial ARK driver. Should at least provide 25.175 and 28.322 MHz
  * dot clocks.
+ * Sep 1996: Support 24bpp, but colors are wrong on ARK1000PV with AT&T
+ *	     DAC. Recognize ARK2000MT.
  */
 
 /* #define INCLUDE_HALVED_CLOCKS */
@@ -24,6 +26,7 @@
 #include <stdio.h>
 #include <string.h>
 #include <sys/mman.h>
+#include <signal.h>
 #include "vga.h"
 #include "libvga.h"
 #include "driver.h"
@@ -108,7 +111,12 @@ static void nothing(void)
 static void ark_unlock(void)
 {
     /* Set bit 0 of SR1D. */
-    outSR(0x1D, inSR(0x1D) | 0x01);
+    __svgalib_outSR(0x1D, __svgalib_inSR(0x1D) | 0x01);
+    /*
+     * Also enable writing to CRTC 0-7. This lock seems to have side effects
+     * on some extended registers on the ARK1000PV!
+     */
+    __svgalib_outCR(0x11, __svgalib_inCR(0x11) & ~0x80);
 }
 
 /* Fill in chipset specific mode information */
@@ -121,7 +129,7 @@ static void ark_getmodeinfo(int mode, vga_modeinfo * modeinfo)
 #else
     if (mode < G640x480x256 || mode == G720x348x2) {
 #endif
-	vga_driverspecs.getmodeinfo(mode, modeinfo);
+	__svgalib_vga_driverspecs.getmodeinfo(mode, modeinfo);
 	return;
     }
     switch (modeinfo->colors) {
@@ -138,7 +146,7 @@ static void ark_getmodeinfo(int mode, vga_modeinfo * modeinfo)
     modeinfo->haveblit = 0;
     modeinfo->flags &= ~HAVE_RWPAGE;
     modeinfo->flags |= CAPABLE_LINEAR;
-    if ((inSR(0x12) & 0x03) != 0)
+    if ((__svgalib_inSR(0x12) & 0x03) != 0)
 	modeinfo->flags |= IS_LINEAR;
 }
 
@@ -157,17 +165,17 @@ static int ark_modeavailable(int mode)
 #else
     if (mode < G640x480x256 || mode == G720x348x2)
 #endif
-	return vga_driverspecs.modeavailable(mode);
+	return __svgalib_vga_driverspecs.modeavailable(mode);
 
     /* Enough memory? */
     info = &__svgalib_infotable[mode];
     if (ark_memory * 1024 < info->ydim * info->xbytes)
 	return 0;
 
-    modeinfo = createModeInfoStructureForSvgalibMode(mode);
+    modeinfo = __svgalib_createModeInfoStructureForSvgalibMode(mode);
 
     modetiming = malloc(sizeof(ModeTiming));
-    if (getmodetiming(modetiming, modeinfo, cardspecs)) {
+    if (__svgalib_getmodetiming(modetiming, modeinfo, cardspecs)) {
 	free(modetiming);
 	free(modeinfo);
 	return 0;
@@ -184,24 +192,24 @@ static int ark_saveregs(unsigned char regs[])
     ark_unlock();
 
     /* Save extended registers. */
-    regs[ARK_SR10] = inSR(0x10);
-    regs[ARK_SR11] = inSR(0x11);
-    regs[ARK_SR12] = inSR(0x12);
-    regs[ARK_SR13] = inSR(0x13);
-    regs[ARK_SR14] = inSR(0x14);
-    regs[ARK_SR15] = inSR(0x15);
-    regs[ARK_SR16] = inSR(0x16);
-    regs[ARK_SR18] = inSR(0x18);
-    regs[ARK_SR19] = inSR(0x19);
-    regs[ARK_SR1C] = inSR(0x1C);
-    regs[ARK_SR1D] = inSR(0x1D);
+    regs[ARK_SR10] = __svgalib_inSR(0x10);
+    regs[ARK_SR11] = __svgalib_inSR(0x11);
+    regs[ARK_SR12] = __svgalib_inSR(0x12);
+    regs[ARK_SR13] = __svgalib_inSR(0x13);
+    regs[ARK_SR14] = __svgalib_inSR(0x14);
+    regs[ARK_SR15] = __svgalib_inSR(0x15);
+    regs[ARK_SR16] = __svgalib_inSR(0x16);
+    regs[ARK_SR18] = __svgalib_inSR(0x18);
+    regs[ARK_SR19] = __svgalib_inSR(0x19);
+    regs[ARK_SR1C] = __svgalib_inSR(0x1C);
+    regs[ARK_SR1D] = __svgalib_inSR(0x1D);
 
-    regs[ARK_CR40] = inCR(0x40);
-    regs[ARK_CR41] = inCR(0x41);
-    regs[ARK_CR42] = inCR(0x42);
-    regs[ARK_CR44] = inCR(0x44);
-    regs[ARK_CR46] = inCR(0x46);
-    regs[ARK_CR50] = inCR(0x50);
+    regs[ARK_CR40] = __svgalib_inCR(0x40);
+    regs[ARK_CR41] = __svgalib_inCR(0x41);
+    regs[ARK_CR42] = __svgalib_inCR(0x42);
+    regs[ARK_CR44] = __svgalib_inCR(0x44);
+    regs[ARK_CR46] = __svgalib_inCR(0x46);
+    regs[ARK_CR50] = __svgalib_inCR(0x50);
 
     inb(0x3C8);
     regs[ARK_PELMASK] = inb(0x3C6);
@@ -219,29 +227,29 @@ static void ark_setregs(const unsigned char regs[], int mode)
 
     /* Write extended registers. */
 
-    outCR(0x46, inCR(0x46) | 0x20);	/* Disable Clock Select Latch. */
+    __svgalib_outCR(0x46, __svgalib_inCR(0x46) | 0x20);	/* Disable Clock Select Latch. */
 
-    outSR(0x10, regs[ARK_SR10]);
-    outSR(0x11, regs[ARK_SR11]);
-    outSR(0x12, regs[ARK_SR12]);
-    outSR(0x13, regs[ARK_SR13]);
-    outSR(0x14, regs[ARK_SR14]);
-    outSR(0x15, regs[ARK_SR15]);
-    outSR(0x16, regs[ARK_SR16]);
-    outSR(0x18, regs[ARK_SR18]);
-    outSR(0x1C, regs[ARK_SR1C]);
-    outSR(0x1D, regs[ARK_SR1D]);
+    __svgalib_outSR(0x10, regs[ARK_SR10]);
+    __svgalib_outSR(0x11, regs[ARK_SR11]);
+    __svgalib_outSR(0x12, regs[ARK_SR12]);
+    __svgalib_outSR(0x13, regs[ARK_SR13]);
+    __svgalib_outSR(0x14, regs[ARK_SR14]);
+    __svgalib_outSR(0x15, regs[ARK_SR15]);
+    __svgalib_outSR(0x16, regs[ARK_SR16]);
+    __svgalib_outSR(0x18, regs[ARK_SR18]);
+    __svgalib_outSR(0x1C, regs[ARK_SR1C]);
+    __svgalib_outSR(0x1D, regs[ARK_SR1D]);
 
-    outCR(0x40, regs[ARK_CR40]);
-    outCR(0x41, regs[ARK_CR41]);
-    outCR(0x42, regs[ARK_CR42]);
-    outCR(0x44, regs[ARK_CR44]);
+    __svgalib_outCR(0x40, regs[ARK_CR40]);
+    __svgalib_outCR(0x41, regs[ARK_CR41]);
+    __svgalib_outCR(0x42, regs[ARK_CR42]);
+    __svgalib_outCR(0x44, regs[ARK_CR44]);
 
     inb(0x3C8);
     outb(0x3C6, regs[ARK_PELMASK]);
     dac_used->restoreState(regs + ARK_DAC_OFFSET);
 
-    outCR(0x46, regs[ARK_CR46]);
+    __svgalib_outCR(0x46, regs[ARK_CR46]);
 }
 
 
@@ -257,7 +265,7 @@ static void ark_initializemode(unsigned char *moderegs,
     ark_saveregs(moderegs);
 
     /* Set up the standard VGA registers for a generic SVGA. */
-    setup_VGA_registers(moderegs, modetiming, modeinfo);
+    __svgalib_setup_VGA_registers(moderegs, modetiming, modeinfo);
 
     /* Set up the extended register values, including modifications */
     /* of standard VGA registers. */
@@ -336,7 +344,7 @@ static void ark_initializemode(unsigned char *moderegs,
 	moderegs[ARK_CRTC_VERTICAL_OVERFLOW] = val;
 
 	/* Horizontal Overflow. */
-	val = moderegs[ARK_CRTC_VERTICAL_OVERFLOW];
+	val = moderegs[ARK_CRTC_HORIZONTAL_OVERFLOW];
 	val &= 0x07;
 	if ((modetiming->CrtcHTotal / 8 - 5) & 0x100)
 	    val |= 0x80;
@@ -350,7 +358,7 @@ static void ark_initializemode(unsigned char *moderegs,
 	    val |= 0x10;
 	if ((modeinfo->lineWidth / 8) & 0x100)
 	    val |= 0x08;
-	moderegs[ARK_CRTC_VERTICAL_OVERFLOW] = val;
+	moderegs[ARK_CRTC_HORIZONTAL_OVERFLOW] = val;
     }
     /* No interlace, standard character clock. */
     moderegs[ARK_VGA_ENHANCEMENT] &= ~0x34;
@@ -409,7 +417,7 @@ static void ark_initializemode(unsigned char *moderegs,
     if (dac_used->id != NORMAL_DAC) {
 	dac_used->initializeState(&moderegs[ARK_DAC_OFFSET],
 				  modeinfo->bitsPerPixel,
-			   colorbits_to_colormode(modeinfo->bitsPerPixel,
+			   __svgalib_colorbits_to_colormode(modeinfo->bitsPerPixel,
 						  modeinfo->colorBits),
 				  modetiming->pixelClock);
     }
@@ -437,14 +445,14 @@ static int ark_setmode(int mode, int prv_mode)
     if (mode < G640x480x256 || mode == G720x348x2)
 #endif
 	/* Let the standard VGA driver set standard VGA modes. */
-	return vga_driverspecs.setmode(mode, prv_mode);
+	return __svgalib_vga_driverspecs.setmode(mode, prv_mode);
     if (!ark_modeavailable(mode))
 	return 1;
 
-    modeinfo = createModeInfoStructureForSvgalibMode(mode);
+    modeinfo = __svgalib_createModeInfoStructureForSvgalibMode(mode);
 
     modetiming = malloc(sizeof(ModeTiming));
-    if (getmodetiming(modetiming, modeinfo, cardspecs)) {
+    if (__svgalib_getmodetiming(modetiming, modeinfo, cardspecs)) {
 	free(modetiming);
 	free(modeinfo);
 	return 1;
@@ -455,16 +463,16 @@ static int ark_setmode(int mode, int prv_mode)
     free(modetiming);
 
     ark_setregs(moderegs, mode);	/* Set extended regs. */
-    __vga_setregs(moderegs);	/* Set standard regs. */
+    __svgalib_setregs(moderegs);	/* Set standard regs. */
     free(moderegs);
 
-    InitializeAcceleratorInterface(modeinfo);
+    __svgalib_InitializeAcceleratorInterface(modeinfo);
 
-    init_acceleration_specs_for_mode(driverspecs->accelspecs,
+    init_acceleration_specs_for_mode(__svgalib_driverspecs->accelspecs,
 				     modeinfo->bitsPerPixel,
 			  modeinfo->lineWidth / modeinfo->bytesPerPixel);
 
-    arkaccel_init(driverspecs->accelspecs,
+    arkaccel_init(__svgalib_driverspecs->accelspecs,
 		  modeinfo->bitsPerPixel,
 		  modeinfo->lineWidth / modeinfo->bytesPerPixel);
 
@@ -502,8 +510,8 @@ static void ark_setdisplaystart(int address)
     outb(0x3c0, 0x13 + 0x20);	/* select ATC reg 0x13 */
     outb(0x3c0, (inb(0x3c1) & 0xf0) | ((address & 3) << 1));
     /* write sa0-1 to bits 1-2 */
-    outb(CRT_I, 0x40);
-    outb(CRT_D, (address & 0x1C0000) >> 18);	/* sa18-sa20 */
+    outb(__svgalib_CRT_I, 0x40);
+    outb(__svgalib_CRT_D, (address & 0x1C0000) >> 18);	/* sa18-sa20 */
 }
 
 
@@ -513,16 +521,16 @@ static void ark_setdisplaystart(int address)
 static void ark_setlogicalwidth(int width)
 {
     outw(0x3d4, 0x13 + (width >> 3) * 256);	/* lw3-lw11 */
-    outb(CRT_I, 0x41);
-    outb(CRT_D, (inb(CRT_D) & ~0x08) | ((width >> 3) & 0x100));
+    outb(__svgalib_CRT_I, 0x41);
+    outb(__svgalib_CRT_D, (inb(__svgalib_CRT_D) & ~0x08) | ((width >> 3) & 0x100));
 }
 
 static int ark_linear(int op, int param)
 {
     if (op == LINEAR_ENABLE) {
 	int size;
-	outSR(0x13, param >> 16);
-	outSR(0x14, param >> 24);
+	__svgalib_outSR(0x13, param >> 16);
+	__svgalib_outSR(0x14, param >> 24);
 	switch (ark_memory) {
 	case 1024:
 	    size = 1;
@@ -534,13 +542,13 @@ static int ark_linear(int op, int param)
 	default:
 	    size = 3;
 	}
-	outSR(0x12, (inSR(0x12) & ~0x03) | size);
+	__svgalib_outSR(0x12, (__svgalib_inSR(0x12) & ~0x03) | size);
 	return 0;
     }
     if (op == LINEAR_DISABLE) {
-	outSR(0x12, inSR(0x12) & ~0x03);	/* 64K map. */
-	outSR(0x13, 0xA0);	/* Revert to 0xA0000. */
-	outSR(0x14, 0x00);
+	__svgalib_outSR(0x12, __svgalib_inSR(0x12) & ~0x03);	/* 64K map. */
+	__svgalib_outSR(0x13, 0xA0);	/* Revert to 0xA0000. */
+	__svgalib_outSR(0x14, 0x00);
 	return 0;
     }
     if (op == LINEAR_QUERY_BASE) {
@@ -568,7 +576,7 @@ static int ark_linear(int op, int param)
 
 /* Function table (exported) */
 
-DriverSpecs ark_driverspecs =
+DriverSpecs __svgalib_ark_driverspecs =
 {
     ark_saveregs,		/* saveregs */
     ark_setregs,		/* setregs */
@@ -642,19 +650,19 @@ static char *ark_process_option(int option, int mode)
 	old_dac_used = dac_used;
 #ifdef INCLUDE_SIERRA_DAC
 	if (strcasecmp(ptr, "Sierra32K") == 0)
-	    dac_used = &Sierra_32K_methods;
+	    dac_used = &__svgalib_Sierra_32K_methods;
 #endif
 #ifdef INCLUDE_ATT20C490_DAC
 	if (strcasecmp(ptr, "ATT20C490") == 0)
-	    dac_used = &ATT20C490_methods;
+	    dac_used = &__svgalib_ATT20C490_methods;
 #endif
 #ifdef INCLUDE_ATT20C498_DAC
 	if (strcasecmp(ptr, "ATT20C498") == 0)
-	    dac_used = &ATT20C498_methods;
+	    dac_used = &__svgalib_ATT20C498_methods;
 #endif
 #ifdef INCLUDE_NORMAL_DAC
 	if (strcasecmp(ptr, "Normal") == 0)	/* force normal VGA dac */
-	    dac_used = &normal_dac_methods;
+	    dac_used = &__svgalib_normal_dac_methods;
 #endif
 	if (old_dac_used != dac_used)
 	    dac_used->initialize();
@@ -686,17 +694,17 @@ static int ark_init(int force, int par1, int par2)
 	ark_memory = par2;
     } else {
 	unsigned char id, val;
-	id = inCR(0x50) >> 3;
+	id = __svgalib_inCR(0x50) >> 3;
 	if (id == 0x12)
 	    ark_chip = ARK1000PV;
-	else if (id == 0x13)
+	else if ((id == 0x13) || (id == 0x14) || (id == 0x20))
 	    ark_chip = ARK2000PV;
 	else {
 	    printf("svgalib: ark: Unknown chiptype %d.\n",
 		   id);
 	    return -1;
 	}
-	val = inSR(0x10);
+	val = __svgalib_inSR(0x10);
 	if (ark_chip == ARK1000PV)
 	    if ((val & 0x40) == 0)
 		ark_memory = 1024;
@@ -725,7 +733,7 @@ static int ark_init(int force, int par1, int par2)
 	/* Not supported. */
 	printf("svgalib: ark: Assuming normal VGA DAC.\n");
 #ifdef INCLUDE_NORMAL_DAC
-	dac_used = &normal_dac_methods;
+	dac_used = &__svgalib_normal_dac_methods;
 #else
 	printf("svgalib: Alas, normal VGA DAC support is not compiled in, goodbye.\n");
 	return 1;
@@ -776,8 +784,9 @@ static int ark_init(int force, int par1, int par2)
 /* end: Initialize cardspecs. */
 
 /* Initialize accelspecs structure. */
-    ark_driverspecs.accelspecs = malloc(sizeof(AccelSpecs));
-    ark_driverspecs.accelspecs->flags = ACCELERATE_ANY_LINEWIDTH;
+    __svgalib_ark_driverspecs.accelspecs = malloc(sizeof(AccelSpecs));
+    __svgalib_clear_accelspecs(__svgalib_ark_driverspecs.accelspecs);
+    __svgalib_ark_driverspecs.accelspecs->flags = ACCELERATE_ANY_LINEWIDTH;
     /* Map memory-mapped I/O register space. */
     mmio_base = valloc(4096);
     mmap(mmio_base, 4096, PROT_WRITE, MAP_FIXED | MAP_SHARED,
@@ -785,14 +794,14 @@ static int ark_init(int force, int par1, int par2)
 
     if (__svgalib_driver_report) {
 	char *bustype;
-	if (inSR(0x19) & 0x80) {
+	if (__svgalib_inSR(0x19) & 0x80) {
 	    ark_bus = VESA;
 	    bustype = "VL bus";
 	} else {
 	    ark_bus = PCI;
 	    bustype = "PCI";
-	    ark_baseaddress = (inSR(0x13) << 16) +
-		(inSR(0x14) << 24);
+	    ark_baseaddress = (__svgalib_inSR(0x13) << 16) +
+		(__svgalib_inSR(0x14) << 24);
 	}
 	printf("svgalib: Using ARK driver (%s, %dK, %s).",
 	       ark_chipname[ark_chip], ark_memory, bustype);
@@ -800,7 +809,7 @@ static int ark_init(int force, int par1, int par2)
 	    printf(" Base address = 0x%08X.", ark_baseaddress);
 	printf("\n");
     }
-    driverspecs = &ark_driverspecs;
+    __svgalib_driverspecs = &__svgalib_ark_driverspecs;
 
     return 0;
 }
@@ -973,7 +982,7 @@ static void arkaccel_init(AccelSpecs * accelspecs, int bpp, int width_in_pixels)
 	pitch = 6;
 	break;
     }
-    outSR(0x17, (inSR(0x17) & ~0x07) | pitch);
+    __svgalib_outSR(0x17, (__svgalib_inSR(0x17) & ~0x07) | pitch);
 #else
     SETBITMAPCONFIG(LINEARSTENCILADDR | LINEARSOURCEADDR |
 		    LINEARDESTADDR);
@@ -981,10 +990,10 @@ static void arkaccel_init(AccelSpecs * accelspecs, int bpp, int width_in_pixels)
 }
 
 #define FINISHBACKGROUNDBLITS() \
-	if (accel_mode & BLITS_IN_BACKGROUND) \
+	if (__svgalib_accel_mode & BLITS_IN_BACKGROUND) \
 		WAITUNTILFINISHED();
 
-void arkaccel_FillBox(int x, int y, int width, int height)
+void __svgalib_arkaccel_FillBox(int x, int y, int width, int height)
 {
     int destaddr;
     destaddr = BLTPIXADDRESS(x, y);
@@ -994,11 +1003,11 @@ void arkaccel_FillBox(int x, int y, int width, int height)
     SETHEIGHT(height);
     SETCOMMAND(SELECTBGCOLOR | SELECTFGCOLOR | STENCILONES |
 	       DISABLEPLANEMASK | DISABLECLIPPING | BITBLT);
-    if (!(accel_mode & BLITS_IN_BACKGROUND))
+    if (!(__svgalib_accel_mode & BLITS_IN_BACKGROUND))
 	WAITUNTILFINISHED();
 }
 
-void arkaccel_coords_FillBox(int x, int y, int width, int height)
+void __svgalib_arkaccel_coords_FillBox(int x, int y, int width, int height)
 {
     FINISHBACKGROUNDBLITS();
     SETDESTXY(x, y);
@@ -1006,11 +1015,11 @@ void arkaccel_coords_FillBox(int x, int y, int width, int height)
     SETHEIGHT(height);
     SETCOMMAND(SELECTBGCOLOR | SELECTFGCOLOR | STENCILONES |
 	       DISABLEPLANEMASK | DISABLECLIPPING | BITBLT);
-    if (!(accel_mode & BLITS_IN_BACKGROUND))
+    if (!(__svgalib_accel_mode & BLITS_IN_BACKGROUND))
 	WAITUNTILFINISHED();
 }
 
-void arkaccel_ScreenCopy(int x1, int y1, int x2, int y2, int width,
+void __svgalib_arkaccel_ScreenCopy(int x1, int y1, int x2, int y2, int width,
 			 int height)
 {
     int srcaddr, destaddr, dir;
@@ -1019,8 +1028,8 @@ void arkaccel_ScreenCopy(int x1, int y1, int x2, int y2, int width,
     dir = RIGHTANDDOWN;
     if ((y1 < y2 || (y1 == y2 && x1 < x2))
 	&& y1 + height > y2) {
-	srcaddr += (height - 1) * accel_screenpitch + width - 1;
-	destaddr += (height - 1) * accel_screenpitch + width - 1;
+	srcaddr += (height - 1) * __svgalib_accel_screenpitch + width - 1;
+	destaddr += (height - 1) * __svgalib_accel_screenpitch + width - 1;
 	dir = LEFTANDUP;
     }
     FINISHBACKGROUNDBLITS();
@@ -1031,11 +1040,11 @@ void arkaccel_ScreenCopy(int x1, int y1, int x2, int y2, int width,
     SETCOMMAND(BACKGROUNDBITMAP | FOREGROUNDBITMAP |
 	       STENCILONES | DISABLEPLANEMASK | DISABLECLIPPING |
 	       BITBLT | dir);
-    if (!(accel_mode & BLITS_IN_BACKGROUND))
+    if (!(__svgalib_accel_mode & BLITS_IN_BACKGROUND))
 	WAITUNTILFINISHED();
 }
 
-void arkaccel_coords_ScreenCopy(int x1, int y1, int x2, int y2, int width,
+void __svgalib_arkaccel_coords_ScreenCopy(int x1, int y1, int x2, int y2, int width,
 				int height)
 {
     int dir;
@@ -1056,11 +1065,11 @@ void arkaccel_coords_ScreenCopy(int x1, int y1, int x2, int y2, int width,
     SETCOMMAND(BACKGROUNDBITMAP | FOREGROUNDBITMAP |
 	       STENCILONES | DISABLEPLANEMASK | DISABLECLIPPING |
 	       BITBLT | dir);
-    if (!(accel_mode & BLITS_IN_BACKGROUND))
+    if (!(__svgalib_accel_mode & BLITS_IN_BACKGROUND))
 	WAITUNTILFINISHED();
 }
 
-void arkaccel_DrawHLineList(int ymin, int n, int *xmin, int *xmax)
+void __svgalib_arkaccel_DrawHLineList(int ymin, int n, int *xmin, int *xmax)
 {
     int y, destaddr;
     FINISHBACKGROUNDBLITS();
@@ -1085,14 +1094,14 @@ void arkaccel_DrawHLineList(int ymin, int n, int *xmin, int *xmax)
 	}
 	xmin++;
 	xmax++;
-	destaddr += accel_screenpitch;
+	destaddr += __svgalib_accel_screenpitch;
 	n--;
     }
-    if (!(accel_mode & BLITS_IN_BACKGROUND))
+    if (!(__svgalib_accel_mode & BLITS_IN_BACKGROUND))
 	WAITUNTILFINISHED();
 }
 
-void arkaccel_coords_DrawHLineList(int ymin, int n, int *xmin, int *xmax)
+void __svgalib_arkaccel_coords_DrawHLineList(int ymin, int n, int *xmin, int *xmax)
 {
     int y;
     FINISHBACKGROUNDBLITS();
@@ -1118,22 +1127,22 @@ void arkaccel_coords_DrawHLineList(int ymin, int n, int *xmin, int *xmax)
 	xmax++;
 	n--;
     }
-    if (!(accel_mode & BLITS_IN_BACKGROUND))
+    if (!(__svgalib_accel_mode & BLITS_IN_BACKGROUND))
 	WAITUNTILFINISHED();
 }
 
 
-void arkaccel_SetFGColor(int fg)
+void __svgalib_arkaccel_SetFGColor(int fg)
 {
     SETFOREGROUNDCOLOR(fg);
 }
 
-void arkaccel_SetBGColor(int fg)
+void __svgalib_arkaccel_SetBGColor(int fg)
 {
     SETBACKGROUNDCOLOR(fg);
 }
 
-void arkaccel_PutBitmap(int x, int y, int w, int h, void *bitmap)
+void __svgalib_arkaccel_PutBitmap(int x, int y, int w, int h, void *bitmap)
 {
     int destaddr, count;
 #ifndef COORDINATES
@@ -1156,6 +1165,7 @@ void arkaccel_PutBitmap(int x, int y, int w, int h, void *bitmap)
     SETHEIGHT(h);
     SETBITMAPCONFIG(LINEARSTENCILADDR | LINEARSOURCEADDR |
 		    LINEARDESTADDR | SYSTEMSTENCIL);
+    SIGNALBLOCK;
     SETCOMMAND(SELECTBGCOLOR | SELECTFGCOLOR | STENCILBITMAP
 	       | DISABLEPLANEMASK | DISABLECLIPPING | BITBLT);
     while (count >= 65536) {
@@ -1165,6 +1175,7 @@ void arkaccel_PutBitmap(int x, int y, int w, int h, void *bitmap)
     }
     if (count > 0)
 	memcpy(__svgalib_graph_mem, bitmap, count);
+    SIGNALUNBLOCK;
     WAITUNTILFINISHED();
 }
 
@@ -1177,13 +1188,13 @@ static unsigned char ark_rop_map[] =
     0xA				/* INVERT */
 };
 
-void arkaccel_SetRasterOp(int r)
+void __svgalib_arkaccel_SetRasterOp(int r)
 {
     /* Write rop at bits 0-3 and 8-11. */
     SETCOLORMIXSELECT((int) ark_rop_map[r] * 0x0101);
 }
 
-void arkaccel_Sync(void)
+void __svgalib_arkaccel_Sync(void)
 {
     WAITUNTILFINISHED();
 }
@@ -1199,6 +1210,9 @@ static void init_acceleration_specs_for_mode(AccelSpecs * accelspecs, int bpp,
     accelspecs->operations = 0;
     accelspecs->ropOperations = 0;
     accelspecs->transparencyOperations = 0;
+    accelspecs->ropModes = (1<<ROP_COPY) | (1<<ROP_OR) | (1<<ROP_AND) |
+				(1<<ROP_XOR) | (1<<ROP_INVERT);
+    accelspecs->transparencyModes = 0;
 #ifdef COORDINATES
     if (width_in_pixels != 640 && width_in_pixels != 800
 	&& width_in_pixels != 1024 && width_in_pixels != 1280
@@ -1217,15 +1231,15 @@ static void init_acceleration_specs_for_mode(AccelSpecs * accelspecs, int bpp,
     }
     /* Set the function pointers; availability is handled by flags. */
 #ifdef COORDINATES
-    accelspecs->FillBox = arkaccel_coords_FillBox;
-    accelspecs->ScreenCopy = arkaccel_coords_ScreenCopy;
+    accelspecs->FillBox = __svgalib_arkaccel_coords_FillBox;
+    accelspecs->ScreenCopy = __svgalib_arkaccel_coords_ScreenCopy;
 #else
-    accelspecs->FillBox = arkaccel_FillBox;
-    accelspecs->ScreenCopy = arkaccel_ScreenCopy;
+    accelspecs->FillBox = __svgalib_arkaccel_FillBox;
+    accelspecs->ScreenCopy = __svgalib_arkaccel_ScreenCopy;
 #endif
-    accelspecs->SetFGColor = arkaccel_SetFGColor;
-    accelspecs->SetRasterOp = arkaccel_SetRasterOp;
-    accelspecs->Sync = arkaccel_Sync;
-    accelspecs->SetBGColor = arkaccel_SetBGColor;
-    accelspecs->PutBitmap = arkaccel_PutBitmap;
+    accelspecs->SetFGColor = __svgalib_arkaccel_SetFGColor;
+    accelspecs->SetRasterOp = __svgalib_arkaccel_SetRasterOp;
+    accelspecs->Sync = __svgalib_arkaccel_Sync;
+    accelspecs->SetBGColor = __svgalib_arkaccel_SetBGColor;
+    accelspecs->PutBitmap = __svgalib_arkaccel_PutBitmap;
 }
