@@ -176,6 +176,10 @@
 #include <stdio.h>
 #include <unistd.h>
 #include <stdarg.h>
+#include <string.h>
+#ifdef USEGLIBC
+#include <sys/io.h>
+#endif
 #include "vga.h"
 #include "libvga.h"
 #include "driver.h"
@@ -305,8 +309,8 @@ static int et4000_clocks(int clk)
 
     /* get actual clock */
     res = (inb(MIS_R) >> 2) & 3;	/* bit 0..1 */
-    outb(CRT_I, 0x34);
-    res |= (inb(CRT_D) & 2) << 1;	/* bit 2 */
+    outb(__svgalib_CRT_I, 0x34);
+    res |= (inb(__svgalib_CRT_D) & 2) << 1;	/* bit 2 */
     outb(SEQ_I, 0x07);
     temp = inb(SEQ_D);
     if (temp & 0x41 == 0x41)
@@ -320,9 +324,9 @@ static int et4000_clocks(int clk)
 	/* Set clock */
 	temp = inb(MIS_R) & 0xF3;
 	outb(MIS_W, temp | ((clk & 3) << 2));
-	outb(CRT_I, 0x34);
-	temp = inb(CRT_D) & 0xFD;
-	outb(CRT_D, temp | ((clk & 4) >> 1));
+	outb(__svgalib_CRT_I, 0x34);
+	temp = inb(__svgalib_CRT_D) & 0xFD;
+	outb(__svgalib_CRT_D, temp | ((clk & 4) >> 1));
 	outb(SEQ_I, 0x07);
 	temp = inb(SEQ_D) & 0xBE;
 	temp |= (clk & 0x10 ? 0x41 : 0x00);
@@ -445,11 +449,11 @@ static int et4000_saveregs(unsigned char regs[])
     et4000_unlock();
     /* save extended CRT registers */
     for (i = 0; i < 8; i++) {
-	port_out(0x30 + i, CRT_I);
-	regs[EXT + i] = port_in(CRT_D);
+	port_out(0x30 + i, __svgalib_CRT_I);
+	regs[EXT + i] = port_in(__svgalib_CRT_D);
     }
-    port_out(0x3f, CRT_I);
-    regs[EXT + 8] = port_in(CRT_D);
+    port_out(0x3f, __svgalib_CRT_I);
+    regs[EXT + 8] = port_in(__svgalib_CRT_D);
 
     /* save extended sequencer register */
     port_out(7, SEQ_I);
@@ -460,7 +464,7 @@ static int et4000_saveregs(unsigned char regs[])
     regs[EXT + 11] = port_in(0x3cd);
 
     /* save extended attribute register */
-    port_in(IS1_R);		/* reset flip flop */
+    port_in(__svgalib_IS1_R);		/* reset flip flop */
     port_out(0x16, ATT_IW);
     regs[EXT + 12] = port_in(ATT_R);
 
@@ -506,36 +510,36 @@ static void et4000_setregs(const unsigned char regs[], int mode)
 #if 0				/* This writes to registers we shouldn't write to. */
     /* write extended CRT registers */
     for (i = 0; i < 6; i++) {
-	port_out(0x32 + i, CRT_I);
-	port_out(regs[EXT + i], CRT_D);
+	port_out(0x32 + i, __svgalib_CRT_I);
+	port_out(regs[EXT + i], __svgalib_CRT_D);
     }
-    port_out(0x3f, CRT_I);
-    port_out(regs[EXT + 6], CRT_D);
+    port_out(0x3f, __svgalib_CRT_I);
+    port_out(regs[EXT + 6], __svgalib_CRT_D);
 #endif
 
     /* deprotect CRT register 0x35 */
-    port_out(0x11, CRT_I);
-    save = port_in(CRT_D);
-    port_out(save & 0x7F, CRT_D);
+    port_out(0x11, __svgalib_CRT_I);
+    save = port_in(__svgalib_CRT_D);
+    port_out(save & 0x7F, __svgalib_CRT_D);
 
     /* write extended CRT registers */
     for (i = 0; i < 6; i++) {
-	port_out(0x30 + i, CRT_I);
-	port_out(regs[EXT + i], CRT_D);
+	port_out(0x30 + i, __svgalib_CRT_I);
+	port_out(regs[EXT + i], __svgalib_CRT_D);
     }
-    port_out(0x36, CRT_I);
-    port_out((port_in(CRT_D) & 0x40) | (regs[EXT + 6] & 0xbf), CRT_D);
-    port_out(0x37, CRT_I);
-    port_out((port_in(CRT_D) & 0xbb) | (regs[EXT + 7] & 0x44), CRT_D);
-    port_out(0x3f, CRT_I);
-    port_out(regs[EXT + 8], CRT_D);
+    port_out(0x36, __svgalib_CRT_I);
+    port_out((port_in(__svgalib_CRT_D) & 0x40) | (regs[EXT + 6] & 0xbf), __svgalib_CRT_D);
+    port_out(0x37, __svgalib_CRT_I);
+    port_out((port_in(__svgalib_CRT_D) & 0xbb) | (regs[EXT + 7] & 0x44), __svgalib_CRT_D);
+    port_out(0x3f, __svgalib_CRT_I);
+    port_out(regs[EXT + 8], __svgalib_CRT_D);
 
     /* set original CRTC 0x11 */
-    port_out(0x11, CRT_I);
-    port_out(save, CRT_D);
+    port_out(0x11, __svgalib_CRT_I);
+    port_out(save, __svgalib_CRT_D);
 
     /* write extended attribute register */
-    port_in(IS1_R);		/* reset flip flop */
+    port_in(__svgalib_IS1_R);		/* reset flip flop */
     port_out(0x16, ATT_IW);
     port_out(regs[EXT + 12], ATT_IW);
 
@@ -564,7 +568,7 @@ static int et4000_modeavailable(int mode)
 
     regs = LOOKUPMODE(et4000_modes, mode);
     if (regs == NULL || mode == GPLANE16)
-	return vga_driverspecs.modeavailable(mode);
+	return __svgalib_vga_driverspecs.modeavailable(mode);
     if (regs == DISABLE_MODE || mode <= TEXT || mode > GLASTMODE)
 	return 0;
 
@@ -621,18 +625,18 @@ static int et4000_setmode(int mode, int prv_mode)
 
     if (et4000_dac)
 	/* Standard dac behaviour */
-	__vga_hicolor(et4000_dac, STD_DAC);
+	__svgalib_hicolor(et4000_dac, STD_DAC);
     switch (et4000_modeavailable(mode)) {
     case STDVGADRV:
 	/* Reset extended register that is set to non-VGA */
 	/* compatible value for 132-column textmodes (at */
 	/* least on some cards). */
 	et4000_unlock();
-	outb(CRT_I, 0x34);
-	i = inb(CRT_D);
+	outb(__svgalib_CRT_I, 0x34);
+	i = inb(__svgalib_CRT_D);
 	if ((i & 0x02) == 0x02)
-	    outb(CRT_D, (i & 0xfe));
-	return vga_driverspecs.setmode(mode, prv_mode);
+	    outb(__svgalib_CRT_D, (i & 0xfe));
+	return __svgalib_vga_driverspecs.setmode(mode, prv_mode);
     case SVGADRV:
 	regs = LOOKUPMODE(et4000_modes, mode);
 	if (regs != NULL)
@@ -644,16 +648,16 @@ static int et4000_setmode(int mode, int prv_mode)
     if (et4000_dac && !et4000_extdac)
 	switch (vga_getmodeinfo(mode)->colors) {
 	case 1 << 15:
-	    __vga_hicolor(et4000_dac, HI15_DAC);
+	    __svgalib_hicolor(et4000_dac, HI15_DAC);
 	    break;
 	case 1 << 16:
-	    __vga_hicolor(et4000_dac, HI16_DAC);
+	    __svgalib_hicolor(et4000_dac, HI16_DAC);
 	    break;
 	case 1 << 24:
-	    __vga_hicolor(et4000_dac, TC24_DAC);
+	    __svgalib_hicolor(et4000_dac, TC24_DAC);
 	    break;
 	}
-    __vga_setregs(regs);
+    __svgalib_setregs(regs);
     et4000_setregs(regs, mode);
     return 0;
 }
@@ -877,7 +881,7 @@ static int et4000_linear(int op, int param)
 
 /* Function table (exported) */
 
-DriverSpecs et4000_driverspecs =
+DriverSpecs __svgalib_et4000_driverspecs =
 {
     et4000_saveregs,
     et4000_setregs,
@@ -1012,7 +1016,7 @@ static int et4000_init(int force, int par1, int par2)
 	regs = fopen(ET4000_REGS, "r");
 	if (regs != 0) {
 	    et4000_modes = NULL;
-	    __vga_readmodes(regs, &et4000_modes, &et4000_dac, &clocks[0]);
+	    __svgalib_readmodes(regs, &et4000_modes, &et4000_dac, &clocks[0]);
 	    fclose(regs);
 	} else
 	    et4000_modes = &No_Modes;
@@ -1130,7 +1134,7 @@ static int et4000_init(int force, int par1, int par2)
 #endif
 	printf("\n");
     }
-    driverspecs = &et4000_driverspecs;
+    __svgalib_driverspecs = &__svgalib_et4000_driverspecs;
 
     pos_ext_settings = 0;
     if (et4000_dac & (8 | 16 | 32 | 64))
