@@ -7,6 +7,8 @@
 #ifndef _LIBVGA_H
 #define _LIBVGA_H
 
+#include <string.h>
+
 /* --------------------- Macro definitions shared by library modules */
 
 /* VGA index register ports */
@@ -58,14 +60,14 @@
 #define EXT     (MIS+MIS_C)	/* SVGA Extended Registers */
 
 /* Shorthands for chipset (driver) specific calls */
-#define chipset_saveregs driverspecs->saveregs
-#define chipset_setregs driverspecs->setregs
-#define chipset_unlock driverspecs->unlock
-#define chipset_test driverspecs->test
-#define chipset_setpage driverspecs->setpage
-#define chipset_setmode driverspecs->setmode
-#define chipset_modeavailable driverspecs->modeavailable
-#define chipset_getmodeinfo driverspecs->getmodeinfo
+#define chipset_saveregs __svgalib_driverspecs->saveregs
+#define chipset_setregs __svgalib_driverspecs->setregs
+#define chipset_unlock __svgalib_driverspecs->unlock
+#define chipset_test __svgalib_driverspecs->test
+#define chipset_setpage __svgalib_driverspecs->__svgalib_setpage
+#define chipset_setmode __svgalib_driverspecs->setmode
+#define chipset_modeavailable __svgalib_driverspecs->modeavailable
+#define chipset_getmodeinfo __svgalib_driverspecs->getmodeinfo
 
 /* Shorthands for internal variables and functions */
 #define CI	__svgalib_cur_info
@@ -122,7 +124,7 @@
 #define FONT_BASE  0xA0000
 #endif
 #define GRAPH_SIZE 0x10000
-#define FONT_SIZE  0x2000
+#define FONT_SIZE  (0x2000 * 4) /* 2.0.x kernel can use 2 512 char. fonts */
 #define GPLANE16   G640x350x16
 
 /* graphics mode information */
@@ -136,9 +138,9 @@ struct info {
 
 /* --------------------- Variable definitions shared by library modules */
 
-extern int CRT_I;		/* current CRT index register address */
-extern int CRT_D;		/* current CRT data register address */
-extern int IS1_R;		/* current input status register address */
+extern int __svgalib_CRT_I;		/* current CRT index register address */
+extern int __svgalib_CRT_D;		/* current CRT data register address */
+extern int __svgalib_IS1_R;		/* current input status register address */
 extern struct info CI;		/* current video parameters */
 extern int COL;			/* current color            */
 extern int CM;			/* current video mode       */
@@ -153,44 +155,51 @@ extern int MODEFLAGS;		/* copy of flags of current modeinfo->flags */
 
 extern int __svgalib_mem_fd;
 extern int __svgalib_tty_fd;
-extern int __svgalib_console_fd;
 extern int __svgalib_mouse_fd;
-extern int kbd_fd;
+extern int __svgalib_kbd_fd;
+extern int __svgalib_runinbackground;
 
 /* --------------------- Function definitions shared by library modules */
 
-extern int __vga_setregs(const unsigned char *regs);
-extern int __vga_saveregs(unsigned char *regs);
-extern void __vga_dumpregs(const unsigned char regs[], int n);
-extern void __vga_get_perm(void);
-extern int __vga_getchipset(void);
-extern int __vga_name2number(char *modename);
-extern void __vga_delay(void);
-extern int __vga_addmode(int xdim, int ydim, int cols, int xbytes, int bytespp);
+extern int __svgalib_setregs(const unsigned char *regs);
+extern int __svgalib_saveregs(unsigned char *regs);
+extern void __svgalib_dumpregs(const unsigned char regs[], int n);
+extern void __svgalib_get_perm(void);
+extern int __svgalib_getchipset(void);
+extern int __svgalib_name2number(char *modename);
+extern void __svgalib_delay(void);
+extern int __svgalib_addmode(int xdim, int ydim, int cols, int xbytes, int bytespp);
 extern int __svgalib_physmem(void);
 extern void __svgalib_waitvtactive(void);
+extern void __svgalib_open_devconsole(void);
 extern void __svgalib_flipaway(void);
 
+#if 0 
+/* remove this part ? */
+extern void __svgalib_releasevt_signal(int n);
+extern void __svgalib_acquirevt_signal(int n);
+#endif
+
 #ifdef __alpha__
 
 #include <asm/io.h>
 
-static inline void port_out(int value, int port)
+static __inline__ void port_out(int value, int port)
 {
     _outb(value, port);
 }
 
-static inline void port_outw(int value, int port)
+static __inline__ void port_outw(int value, int port)
 {
     _outw(value, port);
 }
 
-static inline int port_in(int port)
+static __inline__ int port_in(int port)
 {
     return _inb(port);
 }
 
-static inline int port_inw(int port)
+static __inline__ int port_inw(int port)
 {
     return _inw(port);
 }
@@ -225,25 +234,25 @@ extern void vga_writew(unsigned short b, unsigned long base, unsigned long off);
 
 #else
 
-static inline void port_out(int value, int port)
+static __inline__ void port_out(int value, int port)
 {
     __asm__ volatile ("outb %0,%1"
 	      ::"a" ((unsigned char) value), "d"((unsigned short) port));
 }
 
-static inline void port_outw(int value, int port)
+static __inline__ void port_outw(int value, int port)
 {
     __asm__ volatile ("outw %0,%1"
 	     ::"a" ((unsigned short) value), "d"((unsigned short) port));
 }
 
-static inline void port_outl(int value, int port)
+static __inline__ void port_outl(int value, int port)
 {
     __asm__ volatile ("outl %0,%1"
              ::"a" ((unsigned long)value), "d" ((unsigned short) port));
 }
 
-static inline int port_in(int port)
+static __inline__ int port_in(int port)
 {
     unsigned char value;
     __asm__ volatile ("inb %1,%0"
@@ -252,7 +261,7 @@ static inline int port_in(int port)
     return value;
 }
 
-static inline int port_inw(int port)
+static __inline__ int port_inw(int port)
 {
     unsigned short value;
     __asm__ volatile ("inw %1,%0"
@@ -261,7 +270,7 @@ static inline int port_inw(int port)
     return value;
 }
 
-static inline int port_inl(int port)
+static __inline__ int port_inl(int port)
 {
     unsigned int value;
     __asm__ volatile("inl %1,%0" :
@@ -282,11 +291,26 @@ static inline int port_inl(int port)
 
 /* Note that the arguments of outb/w are reversed compared with the */
 /* kernel sources. The XFree86 drivers also use this format. */
+#undef inb
+#undef inw
+#undef inl
+#undef outb
+#undef outw
+#undef outl
+
 #define inb port_in
 #define inw port_inw
 #define inl port_inl
 #define outb(port, value) port_out(value, port)
 #define outw(port, value) port_outw(value, port)
 #define outl(port, value) port_outl(value, port)
+
+/* Background things */
+
+extern unsigned char *__svgalib_give_graph_red(void);
+extern unsigned char *__svgalib_give_graph_green(void);
+extern unsigned char *__svgalib_give_graph_blue(void);
  
 #endif
+
+#define zero_sa_mask(maskptr) memset(maskptr, 0, sizeof(sigset_t))
