@@ -10,7 +10,10 @@
  */
 
 /*
- * Last modified: 21 Feb 1996
+ * Last modified: 1 Sep 1997
+ *
+ * Sep 1997 (Greg Alexander):
+ * Recognizes S3Trio64V2/DX cards as Trio64's.
  *
  * Feb 1996 (Stephen Lee):
  * 968/IBMRGB support.  Only 256 colors for now.
@@ -86,6 +89,9 @@
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>		/* iopl() */
+#ifdef USEGLIBC
+#include <sys/io.h>             /* glibc iopl() */
+#endif
 #include "vga.h"
 #include "libvga.h"
 #include "driver.h"
@@ -129,14 +135,14 @@
 
 enum {
     S3_911, S3_924, S3_801, S3_805, S3_928, S3_864, S3_964, S3_TRIO32,
-    S3_TRIO64, S3_866, S3_868, S3_968
+    S3_TRIO64, S3_866, S3_868, S3_968, S3_765
 };
 
 static const char *s3_chipname[] =
 {"911", "924", "801", "805", "928",
- "864", "964", "Trio32", "Trio64", "866", "868", "968"};
+ "864", "964", "Trio32", "Trio64", "866", "868", "968", "Trio64V+"};
 
-#define S3_CR(n)	EXT + (0x##n) - 0x30
+#define S3_CR(n)	(EXT + (0x##n) - 0x30)
 
 #define S3_CR30		S3_CR(30)
 #define S3_CR31		S3_CR(31)
@@ -198,8 +204,26 @@ static const char *s3_chipname[] =
 #define S3_CR64		S3_CR(64)
 #define S3_CR65		S3_CR(65)
 #define S3_CR66		S3_CR(66)
+#define S3_CR6E		S3_CR(6E)
+#define S3_CR6F		S3_CR(6F)
 
-#define S3_8514_OFFSET	(S3_CR6D + 1)
+/* Trio extended SR registers */
+
+#define S3_SR(n)	(S3_CR6F + 1 + (0x##n) - 0x08)
+
+#define S3_SR08		S3_SR(08)
+#define S3_SR09		S3_SR(09)
+#define S3_SR0A		S3_SR(0A)
+#define S3_SR0D		S3_SR(0D)
+#define S3_SR10		S3_SR(10)
+#define S3_SR11		S3_SR(11)
+#define S3_SR12		S3_SR(12)
+#define S3_SR13		S3_SR(13)
+#define S3_SR15		S3_SR(15)
+#define S3_SR18		S3_SR(18)
+#define S3_SR1D		S3_SR(1D)
+
+#define S3_8514_OFFSET	(S3_SR1D + 1)
 
 #define S3_8514_COUNT	(1)	/* number of 2-byte words */
 
@@ -231,7 +255,7 @@ static ClockChipMethods *clk_used;
 static int dac_speed = 0;
 
 /* forward declaration. */
-extern DriverSpecs s3_driverspecs;
+extern DriverSpecs __svgalib_s3_driverspecs;
 
 static int s3_init(int, int, int);
 static void s3_setpage(int page);
@@ -267,14 +291,14 @@ static void nothing(void)
  */
 static void s3_lock(void)
 {
-    outCR(0x39, 0x00);		/* Lock system control regs. */
-    outCR(0x38, 0x00);		/* Lock special regs. */
+    __svgalib_outCR(0x39, 0x00);		/* Lock system control regs. */
+    __svgalib_outCR(0x38, 0x00);		/* Lock special regs. */
 }
 
 static void s3_lock_enh(void)
 {
     if (s3_chiptype > S3_911)
-	outCR(0x40, inCR(0x40) & ~0x01);    /* Lock enhanced command regs. */
+	__svgalib_outCR(0x40, __svgalib_inCR(0x40) & ~0x01);    /* Lock enhanced command regs. */
     s3_lock();
 }
 
@@ -284,15 +308,15 @@ static void s3_lock_enh(void)
  */
 static void s3_unlock(void)
 {
-    outCR(0x38, 0x48);		/* Unlock special regs. */
-    outCR(0x39, 0xA5);		/* Unlock system control regs. */
+    __svgalib_outCR(0x38, 0x48);		/* Unlock special regs. */
+    __svgalib_outCR(0x39, 0xA5);		/* Unlock system control regs. */
 }
 
 static void s3_unlock_enh(void)
 {
     s3_unlock();
     if (s3_chiptype > S3_911)
-	outCR(0x40, inCR(0x40) | 0x01);     /* Unlock enhanced command regs. */
+	__svgalib_outCR(0x40, __svgalib_inCR(0x40) | 0x01);     /* Unlock enhanced command regs. */
 }
 
 /* 
@@ -451,17 +475,17 @@ static int s3_modeavailable(int mode)
     ModeTiming *modetiming;
 
     if (mode < G640x480x256 || mode == G720x348x2)
-	return vga_driverspecs.modeavailable(mode);
+	return __svgalib_vga_driverspecs.modeavailable(mode);
 
     /* Enough memory? */
     info = &__svgalib_infotable[mode];
     if (s3_memory * 1024 < info->ydim * s3_adjlinewidth(info->xbytes))
 	return 0;
 
-    modeinfo = createModeInfoStructureForSvgalibMode(mode);
+    modeinfo = __svgalib_createModeInfoStructureForSvgalibMode(mode);
 
     modetiming = malloc(sizeof(ModeTiming));
-    if (getmodetiming(modetiming, modeinfo, cardspecs)) {
+    if (__svgalib_getmodetiming(modetiming, modeinfo, cardspecs)) {
 	free(modetiming);
 	free(modeinfo);
 	return 0;
@@ -481,15 +505,15 @@ static int s3_saveregs(unsigned char regs[])
     unsigned char b, bmax;
     unsigned char cr38, cr39, cr40;
 
-    cr38 = inCR(0x38);
-    outCR(0x38, 0x48);		/* unlock S3 VGA regs (CR30-CR3B) */
+    cr38 = __svgalib_inCR(0x38);
+    __svgalib_outCR(0x38, 0x48);		/* unlock S3 VGA regs (CR30-CR3B) */
 
-    cr39 = inCR(0x39);
-    outCR(0x39, 0xA5);		/* unlock S3 system control (CR40-CR4F) */
+    cr39 = __svgalib_inCR(0x39);
+    __svgalib_outCR(0x39, 0xA5);		/* unlock S3 system control (CR40-CR4F) */
     /* and extended regs (CR50-CR6D) */
 
-    cr40 = inCR(0x40);		/* unlock enhanced regs */
-    outCR(0x40, cr40 | 0x01);
+    cr40 = __svgalib_inCR(0x40);		/* unlock enhanced regs */
+    __svgalib_outCR(0x40, cr40 | 0x01);
 
     /* retrieve values from private copy */
     memcpy(regs + S3_8514_OFFSET, s3_8514regs, S3_8514_COUNT * 2);
@@ -501,26 +525,31 @@ static int s3_saveregs(unsigned char regs[])
     if (s3_chiptype >= S3_864)
 	bmax = 0x6D;
     for (b = 0x30; b <= bmax; b++)
-	regs[EXT + b - 0x30] = inCR(b);
+	regs[EXT + b - 0x30] = __svgalib_inCR(b);
 
-#if 0
     /* get S3 ext. SR registers */
-    if (s3_chiptype >= S3_864) {
-	regs[EXT + 0x78 - 0x30] = inSR(0x08);
-	outSR(0x08, 0x06);	/* unlock extended seq regs */
-	regs[EXT + 0x79 - 0x30] = inSR(0x09);
-	regs[EXT + 0x7A - 0x30] = inSR(0x0A);
-	regs[EXT + 0x7D - 0x30] = inSR(0x0D);
-	outSR(0x08, regs[EXT + 0x78 - 0x30]);
+    /* if (s3_chiptype >= S3_864) { */
+    if (s3_chiptype == S3_765) {/* SL: actually Trio32/64/V+ */
+	regs[S3_SR08] = __svgalib_inSR(0x08);
+	__svgalib_outSR(0x08, 0x06);	/* unlock extended seq regs */
+	regs[S3_SR09] = __svgalib_inSR(0x09);
+	regs[S3_SR0A] = __svgalib_inSR(0x0A);
+	regs[S3_SR0D] = __svgalib_inSR(0x0D);
+	regs[S3_SR10] = __svgalib_inSR(0x10);
+	regs[S3_SR11] = __svgalib_inSR(0x11);
+	regs[S3_SR12] = __svgalib_inSR(0x12);
+	regs[S3_SR13] = __svgalib_inSR(0x13);
+	regs[S3_SR15] = __svgalib_inSR(0x15);
+	regs[S3_SR18] = __svgalib_inSR(0x18);
+	__svgalib_outSR(0x08, regs[S3_SR08]);
     }
-#endif
 
     dac_used->saveState(regs + S3_DAC_OFFSET);
 
     /* leave the locks the way we found it */
-    outCR(0x40, regs[EXT + 0x40 - 0x30] = cr40);
-    outCR(0x39, regs[EXT + 0x39 - 0x30] = cr39);
-    outCR(0x38, regs[EXT + 0x38 - 0x30] = cr38);
+    __svgalib_outCR(0x40, regs[EXT + 0x40 - 0x30] = cr40);
+    __svgalib_outCR(0x39, regs[EXT + 0x39 - 0x30] = cr39);
+    __svgalib_outCR(0x38, regs[EXT + 0x38 - 0x30] = cr38);
 #if 0
 #include "ramdac/IBMRGB52x.h"
 
@@ -589,25 +618,25 @@ static void s3_setregs(const unsigned char regs[], int mode)
 	bmax = 0x6D;
     for (b = 0x30; b <= bmax; b++) {
 	if (s3_regmask[b - 0x30])
-	    outCR(b, regs[EXT + b - 0x30]);
+	    __svgalib_outCR(b, regs[EXT + b - 0x30]);
     }
 
     if (dac_used->id != NORMAL_DAC) {
 	unsigned char CR1;
 	/* Blank the screen. */
-	CR1 = inCR(0x01);
-	outCR(0x01, CR1 | 0x20);
+	CR1 = __svgalib_inCR(0x01);
+	__svgalib_outCR(0x01, CR1 | 0x20);
 
-	outbCR(0x55, inCR(0x55) | 1);
-	outCR(0x66, regs[S3_CR66]);
-	outCR(0x67, regs[S3_CR67]);	/* S3 pixmux. */
+	__svgalib_outbCR(0x55, __svgalib_inCR(0x55) | 1);
+	__svgalib_outCR(0x66, regs[S3_CR66]);
+	__svgalib_outCR(0x67, regs[S3_CR67]);	/* S3 pixmux. */
 
 	dac_used->restoreState(regs + S3_DAC_OFFSET);
 
-	outCR(0x6D, regs[S3_CR6D]);
-	outbCR(0x55, inCR(0x55) & ~1);
+	__svgalib_outCR(0x6D, regs[S3_CR6D]);
+	__svgalib_outbCR(0x55, __svgalib_inCR(0x55) & ~1);
 
-	outCR(0x01, CR1);	/* Unblank screen. */
+	__svgalib_outCR(0x01, CR1);	/* Unblank screen. */
     }
 #ifdef S3_LINEAR_SUPPORT
     if (mode == TEXT && s3_linear_addr)
@@ -617,9 +646,9 @@ static void s3_setregs(const unsigned char regs[], int mode)
     /* restore CR38/39 (may lock other regs) */
     if (mode == TEXT) {
 	/* restore lock registers as well */
-	outCR(0x40, regs[S3_CR40]);
-	outCR(0x39, regs[S3_CR39]);
-	outCR(0x38, regs[S3_CR38]);
+	__svgalib_outCR(0x40, regs[S3_CR40]);
+	__svgalib_outCR(0x39, regs[S3_CR39]);
+	__svgalib_outCR(0x38, regs[S3_CR38]);
     } else
 	s3_lock_enh();
 }
@@ -635,7 +664,7 @@ static void s3_initializemode(unsigned char *moderegs,
     s3_saveregs(moderegs);
 
     /* Set up the standard VGA registers for a generic SVGA. */
-    setup_VGA_registers(moderegs, modetiming, modeinfo);
+    __svgalib_setup_VGA_registers(moderegs, modetiming, modeinfo);
 
     /* Set up the extended register values, including modifications */
     /* of standard VGA registers. */
@@ -665,7 +694,7 @@ static void s3_initializemode(unsigned char *moderegs,
     if (modeinfo->bitsPerPixel >= 8) {
 	moderegs[S3_CR3A] = 0xB5;
 	if (s3_chiptype == S3_928)
-	    /* ARI: Turn on CHAIN4 for 928, since setup_VGA_registers
+	    /* ARI: Turn on CHAIN4 for 928, since __svgalib_setup_VGA_registers
 							 initializes ModeX */
 	    moderegs[VGA_CR14] = 0x60;
     } else {
@@ -804,14 +833,16 @@ static void s3_initializemode(unsigned char *moderegs,
 		clock = modetiming->pixelClock *
 		    modeinfo->bytesPerPixel;
 	    }
-	    if (s3_memory < 2048 || s3_chiptype == S3_TRIO64)
+	    if (s3_memory < 2048 || s3_chiptype == S3_TRIO64 
+		|| s3_chiptype == S3_765)
 		clock *= 2;
 	    if (s3_chiptype == S3_801 || s3_chiptype == S3_805)
 		mclk = 50000;	/* Assumption. */
 	    else
 		mclk = 60000;	/* Assumption. */
 	    m = (int) ((mclk / 1000.0 * .72 + 16.867) * 89.736 / (clock / 1000.0 + 39) - 21.1543);
-	    if (s3_memory < 2048 || s3_chiptype == S3_TRIO64)
+	    if (s3_memory < 2048 || s3_chiptype == S3_TRIO64
+		|| s3_chiptype == S3_765)
 		m /= 2;
 	    if (m > 31)
 		m = 31;
@@ -825,7 +856,7 @@ static void s3_initializemode(unsigned char *moderegs,
 	    m = 2;
 	else
 	    m = 20;
-
+	
 	moderegs[S3_CR54] = m << 3;
 	moderegs[S3_CR60] = n;
 
@@ -843,11 +874,11 @@ static void s3_initializemode(unsigned char *moderegs,
 		/* Assume CR59/5A are correctly set up for 0xA0000. */
 		/* Set CR6A linear bank to zero. */
 		moderegs[S3_CR6A] &= ~0x3F;
-		/* use alternate setpage() function */
-		s3_driverspecs.setpage = s3_setpage864;
+		/* use alternate __svgalib_setpage() function */
+		__svgalib_s3_driverspecs.__svgalib_setpage = s3_setpage864;
 	    } else {
 		/* doesn't work for 4bpp. */
-		s3_driverspecs.setpage = s3_setpage;
+		__svgalib_s3_driverspecs.__svgalib_setpage = s3_setpage;
 	    }
 	}
 #endif
@@ -923,7 +954,7 @@ static void s3_initializemode(unsigned char *moderegs,
 	moderegs[S3_CR42] |=
 	    modetiming->selectedClockNo;
     }
-    if (s3_chiptype == S3_TRIO64) {
+    if (s3_chiptype == S3_TRIO64 || s3_chiptype == S3_765) {
 	moderegs[S3_CR33] &= ~0x08;
 	if (modeinfo->bitsPerPixel == 16)
 	    moderegs[S3_CR33] |= 0x08;
@@ -934,7 +965,7 @@ static void s3_initializemode(unsigned char *moderegs,
     }
     if (dac_used->id != NORMAL_DAC) {
 	int colormode;
-	colormode = colorbits_to_colormode(modeinfo->bitsPerPixel,
+	colormode = __svgalib_colorbits_to_colormode(modeinfo->bitsPerPixel,
 					   modeinfo->colorBits);
 	dac_used->initializeState(&moderegs[S3_DAC_OFFSET],
 				  modeinfo->bitsPerPixel, colormode,
@@ -1013,7 +1044,7 @@ static void s3_initializemode(unsigned char *moderegs,
 	    moderegs[S3_CR42] |= 0x02;
 	}
     }
-    if (clk_used == &I2061A_clockchip_methods &&
+    if (clk_used == &__svgalib_I2061A_clockchip_methods &&
 	(modetiming->flags & USEPROGRCLOCK)) {
 	/* Clock select. */
 	moderegs[S3_CR42] &= ~0x0F;
@@ -1035,7 +1066,7 @@ static int s3_setmode(int mode, int prv_mode)
 
     if (mode < G640x480x256 || mode == G720x348x2) {
 	/* Let the standard VGA driver set standard VGA modes. */
-	res = vga_driverspecs.setmode(mode, prv_mode);
+	res = __svgalib_vga_driverspecs.setmode(mode, prv_mode);
 	if (res == 0 && s3_chiptype <= S3_928) {
 	    /*
 	     * ARI: Turn off virtual size of 1024 - this fixes all problems
@@ -1044,7 +1075,7 @@ static int s3_setmode(int mode, int prv_mode)
 	     * SL:  Is this for 928 only?  Doesn't matter for 805.
 	     */
 	    s3_unlock();
-	    outCR(0x34, inCR(0x34) & ~0x10);
+	    __svgalib_outCR(0x34, __svgalib_inCR(0x34) & ~0x10);
 	    s3_lock();
 	}
 	return res;
@@ -1052,10 +1083,10 @@ static int s3_setmode(int mode, int prv_mode)
     if (!s3_modeavailable(mode))
 	return 1;
 
-    modeinfo = createModeInfoStructureForSvgalibMode(mode);
+    modeinfo = __svgalib_createModeInfoStructureForSvgalibMode(mode);
 
     modetiming = malloc(sizeof(ModeTiming));
-    if (getmodetiming(modetiming, modeinfo, cardspecs)) {
+    if (__svgalib_getmodetiming(modetiming, modeinfo, cardspecs)) {
 	free(modetiming);
 	free(modeinfo);
 	return 1;
@@ -1068,7 +1099,7 @@ static int s3_setmode(int mode, int prv_mode)
     free(modeinfo);
     free(modetiming);
 
-    __vga_setregs(moderegs);	/* Set standard regs. */
+    __svgalib_setregs(moderegs);	/* Set standard regs. */
     s3_setregs(moderegs, mode);	/* Set extended regs. */
     return 0;
 }
@@ -1226,8 +1257,8 @@ static void s3_setdisplaystart(int address)
  */
 static void s3_setlogicalwidth(int width)
 {
-    outCR(0x13, (width >> 3));	/* lw3-lw11 */
-    outCR(0x51, (width & 0x300) >> 4);	/* lw12-lw13 */
+    __svgalib_outCR(0x13, (width >> 3));	/* lw3-lw11 */
+    __svgalib_outCR(0x51, (width & 0x300) >> 4);	/* lw12-lw13 */
 }
 
 #ifdef S3_LINEAR_SUPPORT
@@ -1244,12 +1275,12 @@ static void s3_setlinear(int addr)
 	if (!m)
 	    m = 1;
 	outb(0x3d5, inb(0x3d5) | 0x10 | m);	/* enable linear mode */
-	outCR(0x59, (addr >> 8) & 0x03);
-	outCR(0x5A, (addr) & 0xFF);
+	__svgalib_outCR(0x59, (addr >> 8) & 0x03);
+	__svgalib_outCR(0x5A, (addr) & 0xFF);
     } else {
 	outb(0x3d5, inb(0x3d5) & ~0x13);	/* disable linear mode */
-	outCR(0x59, 0x00);
-	outCR(0x5A, 0x0A);
+	__svgalib_outCR(0x59, 0x00);
+	__svgalib_outCR(0x5A, 0x0A);
     }
     s3_lock();
 }
@@ -1259,9 +1290,11 @@ static void s3_setlinear(int addr)
 static int s3_linear(int op, int param)
 {
     /* printf("s3_linear(%d,0x%x) called\n", op, param); */
+#if 0    
     if (s3_chiptype != S3_805)
 	return -1;
-    else if (op == LINEAR_QUERY_GRANULARITY) {
+#endif
+    if (op == LINEAR_QUERY_GRANULARITY) {
 	switch (s3_memory) {
 	case 4096:
 	case 2048:
@@ -1293,7 +1326,7 @@ static int s3_linear(int op, int param)
 
 /* Function table (exported) */
 
-DriverSpecs s3_driverspecs =
+DriverSpecs __svgalib_s3_driverspecs =
 {
     s3_saveregs,		/* saveregs */
     s3_setregs,			/* setregs */
@@ -1366,35 +1399,35 @@ static char *s3_process_option(int option, int mode)
 	ptr = strtok(NULL, " ");
 #ifdef INCLUDE_IBMRGB52x_DAC
 	if (strcasecmp(ptr, "IBMRGB52x") == 0)
-	    dac_used = &IBMRGB52x_methods;
+	    dac_used = &__svgalib_IBMRGB52x_methods;
 #endif
 #ifdef INCLUDE_SIERRA_DAC
 	if (strcasecmp(ptr, "Sierra32K") == 0)
-	    dac_used = &Sierra_32K_methods;
+	    dac_used = &__svgalib_Sierra_32K_methods;
 #endif
 #ifdef INCLUDE_SC15025_DAC
 	if (strcasecmp(ptr, "SC15025") == 0)
-	    dac_used = &SC15025_methods;
+	    dac_used = &__svgalib_SC15025_methods;
 #endif
 #ifdef INCLUDE_S3_SDAC_DAC
 	if (strcasecmp(ptr, "SDAC") == 0)
-	    dac_used = &S3_SDAC_methods;
+	    dac_used = &__svgalib_S3_SDAC_methods;
 #endif
 #ifdef INCLUDE_S3_GENDAC_DAC
 	if (strcasecmp(ptr, "GenDAC") == 0)
-	    dac_used = &S3_GENDAC_methods;
+	    dac_used = &__svgalib_S3_GENDAC_methods;
 #endif
 #ifdef INCLUDE_ATT20C490_DAC
 	if (strcasecmp(ptr, "ATT20C490") == 0)
-	    dac_used = &ATT20C490_methods;
+	    dac_used = &__svgalib_ATT20C490_methods;
 #endif
 #ifdef INCLUDE_ATT20C498_DAC
 	if (strcasecmp(ptr, "ATT20C498") == 0)
-	    dac_used = &ATT20C498_methods;
+	    dac_used = &__svgalib_ATT20C498_methods;
 #endif
 #ifdef INCLUDE_NORMAL_DAC
 	if (strcasecmp(ptr, "Normal") == 0)	/* force normal VGA dac */
-	    dac_used = &normal_dac_methods;
+	    dac_used = &__svgalib_normal_dac_methods;
 #endif
 
 	if (clk_used)
@@ -1415,7 +1448,7 @@ static char *s3_process_option(int option, int mode)
 	ptr = strtok(NULL, " \t");
 	if (strcasecmp(ptr, "ICD2061A") == 0 ||
 	    strcasecmp(ptr, "DCS2824") == 0)	/* Diamond, compatible to icd2061a */
-	    clk_used = &I2061A_clockchip_methods;
+	    clk_used = &__svgalib_I2061A_clockchip_methods;
 	clk_used->initialize(cardspecs, dac_used);
 	ptr = strtok(NULL, " \t");
 	if (ptr != NULL) {
@@ -1437,22 +1470,22 @@ static char *s3_process_option(int option, int mode)
 static DacMethods *dacs_to_probe[] =
 {
 #ifdef INCLUDE_S3_SDAC_DAC_TEST
-    &S3_SDAC_methods,
+    &__svgalib_S3_SDAC_methods,
 #endif
 #ifdef INCLUDE_S3_GENDAC_DAC_TEST
-    &S3_GENDAC_methods,
+    &__svgalib_S3_GENDAC_methods,
 #endif
 #ifdef INCLUDE_ATT20C490_DAC_TEST
-    &ATT20C490_methods,
+    &__svgalib_ATT20C490_methods,
 #endif
 #ifdef INCLUDE_SC15025_DAC_TEST
-    &SC15025_methods,
+    &__svgalib_SC15025_methods,
 #endif
 #ifdef INCLUDE_SC1148X_DAC_TEST
-    &SC1148X_methods,
+    &__svgalib_SC1148X_methods,
 #endif
 #ifdef INCLUDE_IBMRGB52x_DAC_TEST
-    &IBMRGB52x_methods,
+    &__svgalib_IBMRGB52x_methods,
 #endif
     NULL};
 
@@ -1463,11 +1496,11 @@ static int s3_init(int force, int par1, int par2)
     s3_unlock();
 
     s3_flags = 0;		/* initialize */
-    id = inCR(0x30);		/* Get chip id. */
+    id = __svgalib_inCR(0x30);		/* Get chip id. */
     rev = id & 0x0F;
     if (id >= 0xE0) {
-	id |= inCR(0x2E) << 8;
-	rev |= inCR(0x2F) << 4;
+	id |= __svgalib_inCR(0x2E) << 8;
+	rev |= __svgalib_inCR(0x2F) << 4;
     }
     if (force) {
 	s3_chiptype = par1;	/* we already know the type */
@@ -1483,7 +1516,7 @@ static int s3_init(int force, int par1, int par2)
 	}
     } else {
 	s3_chiptype = -1;
-	config = inCR(0x36);	/* get configuration info */
+	config = __svgalib_inCR(0x36);	/* get configuration info */
 	switch (id & 0xf0) {
 	case 0x80:
 	    if (rev == 1) {
@@ -1534,8 +1567,13 @@ static int s3_init(int force, int par1, int par2)
 	    case 0x10E0:
 		s3_chiptype = S3_TRIO32;
 		break;
+	    case 0x31E0: /* ViRGE ID */
+	    case 0x01E0: /* S3Trio64V2/DX ... any others? */
 	    case 0x11E0:
-		s3_chiptype = S3_TRIO64;
+		if (rev & 0x0400)
+		    s3_chiptype = S3_765;
+		else
+		    s3_chiptype = S3_TRIO64;
 		break;
 	    case 0x80E0:
 		s3_chiptype = S3_866;
@@ -1616,12 +1654,12 @@ static int s3_init(int force, int par1, int par2)
     __svgalib_read_options(s3_config_options, s3_process_option);
 
 #ifdef INCLUDE_S3_TRIO64_DAC
-    if (s3_chiptype == S3_TRIO64 && dac_used == NULL)
-	dac_used = &Trio64_methods;
+    if ((s3_chiptype == S3_TRIO64 || s3_chiptype == S3_765) && dac_used == NULL)
+	dac_used = &__svgalib_Trio64_methods;
 #endif
 
     if (dac_used == NULL)
-	dac_used = probeDacs(dacs_to_probe);
+	dac_used = __svgalib_probeDacs(dacs_to_probe);
     else
 	dac_used->initialize();
 
@@ -1630,7 +1668,7 @@ static int s3_init(int force, int par1, int par2)
 	/* Not supported. */
 	printf("svgalib: s3: Assuming normal VGA DAC.\n");
 #ifdef INCLUDE_NORMAL_DAC
-	dac_used = &normal_dac_methods;
+	dac_used = &__svgalib_normal_dac_methods;
 	dac_used->initialize();
 #else
 	printf("svgalib: Alas, normal VGA DAC support is not compiled in, goodbye.\n");
@@ -1677,7 +1715,7 @@ static int s3_init(int force, int par1, int par2)
 
 /* end: Initialize cardspecs. */
 
-    driverspecs = &s3_driverspecs;
+    __svgalib_driverspecs = &__svgalib_s3_driverspecs;
 
     return 0;
 }
