@@ -22,6 +22,10 @@
 #include "mouse/vgamouse.h"
 #include "keyboard/vgakeyboard.h"
 
+#ifdef BACKGROUND
+#include "vgabg.h"
+#endif
+
 /* This routine is a pain in my ass and definitely needs to be virtualized..
    M. Weller. (mach32 support) */
 void vga_waitretrace(void)
@@ -44,7 +48,12 @@ void vga_waitretrace(void)
     while (inb(0x3da) & 8);
 }
 
-static void *linearframebuffer;
+#ifndef BACKGROUND
+static void *__svgalib_linearframebuffer;
+#endif
+#ifdef BACKGROUND
+void *__svgalib_linearframebuffer;
+#endif
 
 /*
  * The way IS_LINEAR gets indicated is rather convoluted; if the driver
@@ -62,7 +71,7 @@ unsigned char *
  vga_getgraphmem(void)
 {
     if (vga_getmodeinfo(__svgalib_cur_mode)->flags & IS_LINEAR)
-	return linearframebuffer;
+	return __svgalib_linearframebuffer;
     return __svgalib_graph_mem;
 }
 
@@ -86,6 +95,8 @@ static unsigned char *
  map_framebuffer(unsigned address, int size)
 {
     int mem_fd;
+#ifndef BACKGROUND
+
     unsigned char *fb;
     mem_fd = open("/dev/mem", O_RDWR);
     fb = valloc(size);
@@ -97,10 +108,36 @@ static unsigned char *
 		   mem_fd,
 		   (off_t) address
 	);
-    close(mem_fd);
+    close(mem_fd); /* Never gets here ? */
     /* Using addr 0 (kernel comes up with 0x40000000), 
        and only MAP_SHARED works, but wastes 2Mb 
        of memory. */
+#endif
+#ifdef BACKGROUND
+#if BACKGROUND == 1
+ /* Probably won't go right at first try. */
+ 
+    mem_fd = open("/dev/mem", O_RDWR); 
+    if ((__svgalib_linearframebuffer = valloc(size)) == NULL) {
+	printf("svgalib: allocation error \n");
+	exit(-1);
+    }
+    __svgalib_linear_memory_size=size;
+    __svgalib_graph_mem_linear_check=__svgalib_linearframebuffer;
+    __svgalib_graph_mem_linear_orginal = (unsigned char *) mmap(
+				   (caddr_t) 0,
+				   size,
+				   PROT_READ | PROT_WRITE,
+				   MAP_SHARED,
+				   mem_fd,
+				   (off_t) address
+	);
+	 
+    __svgalib_map_virtual_screen(VIDEOSCREEN); /* Video page */
+    return (__svgalib_graph_mem_linear_orginal);
+    /* close(mem_rd);  Hmm... seems to be opened many times , never closed */
+#endif
+#endif
 }
 
 /*
@@ -113,7 +150,7 @@ static int verify_one_linear_mapping(int base, int size)
 {
     unsigned char *fb;
     int i, result;
-    int (*lfn) (int op, int param) = driverspecs->linear;
+    int (*lfn) (int op, int param) = __svgalib_driverspecs->linear;
     /* Write signatures. */
     for (i = 0; i < size / 65536; i++) {
 	vga_setpage(i);
@@ -174,7 +211,7 @@ static int verify_linear_mapping(int base, int size)
 /* cf. vga_waitretrace, M.Weller */
 int vga_setlinearaddressing(void)
 {
-    int (*lfn) (int op, int param) = driverspecs->linear;
+    int (*lfn) (int op, int param) = __svgalib_driverspecs->linear;
     int memory, mappedMemory;
     vga_modeinfo *modeinfo;
     modeinfo = vga_getmodeinfo(__svgalib_cur_mode);
@@ -227,8 +264,8 @@ int vga_setlinearaddressing(void)
 
       foundlinearframebuffer:
 	(*lfn) (LINEAR_ENABLE, mapaddr);
-	linearframebuffer = map_framebuffer(mapaddr, mappedMemory);
-	if ((long) linearframebuffer == -1) {
+	__svgalib_linearframebuffer = map_framebuffer(mapaddr, mappedMemory);
+	if ((long) __svgalib_linearframebuffer == -1) {
 	    /* Shouldn't happen. */
 	    (*lfn) (LINEAR_DISABLE, mapaddr);
 	    return -1;
@@ -244,7 +281,7 @@ int vga_setlinearaddressing(void)
 	(CAPABLE_LINEAR | EXT_INFO_AVAILABLE)) {
 	if (modeinfo->aperture_size >= modeinfo->memory) {
 	    __svgalib_modeinfo_linearset |= IS_LINEAR;
-	    linearframebuffer = modeinfo->linear_aperture;
+	    __svgalib_linearframebuffer = modeinfo->linear_aperture;
 	    return modeinfo->memory;
 	}
     }
@@ -307,15 +344,15 @@ int vga_waitevent(int which, fd_set * in, fd_set * out, fd_set * except,
     int fd, retval;
 
     if (!in) {
-	FD_ZERO(in);
 	in = &infdset;
+	FD_ZERO(in);
     }
     fd = __svgalib_mouse_fd;	/* __svgalib_mouse_fd might change on
 				   vc switch!! */
     if ((which & VGA_MOUSEEVENT) && (fd >= 0))
 	FD_SET(fd, in);
     if (which & VGA_KEYEVENT) {
-	fd = kbd_fd;
+	fd = __svgalib_kbd_fd;
 	if (fd >= 0) {		/* we are in raw mode */
 	    FD_SET(fd, in);
 	} else {
@@ -334,7 +371,7 @@ int vga_waitevent(int which, fd_set * in, fd_set * out, fd_set * except,
 	}
     }
     if (which & VGA_KEYEVENT) {
-	fd = kbd_fd;
+	fd = __svgalib_kbd_fd;
 	if (fd >= 0) {		/* we are in raw mode */
 	    if (FD_ISSET(fd, in)) {
 		FD_CLR(fd, in);
@@ -349,25 +386,15 @@ int vga_waitevent(int which, fd_set * in, fd_set * out, fd_set * except,
     return retval;
 }
 
+#if 0
 static void reserved_message(void)
 {
     printf("svgalib: Function not present in shared library image.\n");
     printf("         You need a newer version of svgalib.\n");
 }
 
-void vga_reserved19(void)
-{
-    reserved_message();
-}
-void vga_reserved20(void)
-{
-    reserved_message();
-}
-void vga_reserved21(void)
-{
-    reserved_message();
-}
 void vga_reserved22(void)
 {
     reserved_message();
 }
+#endif
